/*

SwapChannels.js: Swap channels in an RGB image
================================================================================

Mit SwapChannel ist eine einfache Vertauschung der Kanäle möglich. Die 3 Kanäle
RGB werden mit ihren Indizes 0;1;2 angezeigt. Setzt man z.B. für G den Index 0
ein, dann werden Grün und Rot vertauscht.
 
With SwapChannel a simple permutation of the channels is possible. The 3 RGB
channels are shown with their indices 0;1;2. Substituting for example, for G
the index 0, then green and red are reversed.
================================================================================


 * Swap Channels Value v1.1
 *
 * Copyright (C) 2012, Hartmut V. Bornemann
 *
 * References: None
 *
 */
#feature-id    Utilities > Swap channels

#include <pjsr/Color.jsh>
#include <pjsr/Sizer.jsh>
#include <pjsr/FrameStyle.jsh>
#include <pjsr/TextAlign.jsh>
#include <pjsr/StdButton.jsh>
#include <pjsr/StdIcon.jsh>
#include <pjsr/StdCursor.jsh>
#include <pjsr/UndoFlag.jsh>
#include <pjsr/SampleType.jsh>
#include <pjsr/ColorSpace.jsh>
#include <pjsr/NumericControl.jsh>

#define TITLE "Swap channels of the active window"
#define VERSION "1.00"

#feature-info Swap RGB to a different order, i.e GRB or BGR.<br>\
   <br>\Written by Hartmut V. Bornemann.<br>\
   <br>\
   References: None<br>\
   <br>\

/*

 */
function swap3(){}

function swap3()
{
   this.__base__ = Dialog;
   this.__base__();

   // Get access to the current active image window.

   var window = ImageWindow.activeWindow;
   var obj = this;
   var oimg = window.currentView.image;

   if (oimg.numberOfChannels != 3)
   {
      MessageBox("The active window must have 3 channels.");
      cancel();
   }

   var labelWidth1 = 100;

   var cIndex = Array(oimg.numberOfChannels);
   var nEdits = Array(oimg.numberOfChannels);

   //

   this.helpLabel = new Label( this );
   with ( this.helpLabel )
   {
      frameStyle = FrameStyle_Box;
      margin = 4;
      wordWrapping = false;
      useRichText = true;
      text = "<p><b>" + TITLE + " v" + VERSION + "</b></p>";
   }

   // Red Channel ----------------------------------------------

   this.R_Channel = new NumericEdit(this);
   with (this.R_Channel) {
      label.text = "R --->";
      label.minWidth = labelWidth1 + 6+1;
      setRange(0, oimg.numberOfChannels-1 );
      setPrecision( 0 );
      setValue( 0);
      toolTip = "<p>Assign channel to red color.</p>";
      onValueUpdated = function( value ) {
         UpdateIndex(obj.R_Channel, nEdits, cIndex);
         }
   }

   this.R_Sizer = new HorizontalSizer;
   with ( this.R_Sizer )
   {
      spacing = 4;
      add( this.R_Channel );
   }

   // Green Channel --------------------------------------------

   this.G_Channel = new NumericEdit(this);
   with (this.G_Channel) {
      label.text = "G --->";
      label.minWidth = labelWidth1 + 6+1;
      setRange(0, oimg.numberOfChannels-1 );
      setPrecision( 0 );
      setValue( 1);
      toolTip = "<p>Assign channel to green color.</p>";
      onValueUpdated = function( value ) {
         UpdateIndex(obj.G_Channel, nEdits, cIndex);
         }
   }

   this.G_Sizer = new HorizontalSizer;
   with ( this.G_Sizer )
   {
      spacing = 4;
      add( this.G_Channel );
   }

   // Blue Channel ---------------------------------------------

   this.B_Channel = new NumericEdit(this);
   with (this.B_Channel) {
      label.text = "B --->";
      label.minWidth = labelWidth1 + 6+1;
      setRange(0, oimg.numberOfChannels-1 );
      setPrecision( 0 );
      setValue( 2);
      toolTip = "<p>Assign channel to blue color.</p>";
      onValueUpdated = function( value ) {
         UpdateIndex(obj.B_Channel, nEdits, cIndex);
         }
   }

   this.B_Sizer = new HorizontalSizer;
   with ( this.B_Sizer )
   {
      spacing = 4;
      add( this.B_Channel );
   }

   // array to store the destination index ---------------------

   cIndex[0] = 0;
   cIndex[1] = 1;
   cIndex[2] = 2;

   // nEdits array allows indexed access to num edit fields ----

   nEdits[0] = this.R_Channel;
   nEdits[1] = this.G_Channel;
   nEdits[2] = this.B_Channel;

   // Buttons                   --------------------------------

   this.ok_Button = new PushButton (this);
   with ( this.ok_Button )
   {
      text = "OK";
      onClick = function()
      {
         // create a new image with swapped channels

         var nimg = new Image(oimg.width,
                              oimg.height,
                              oimg.numberOfChannels,
                              oimg.colorSpace,
                              oimg.bitsPerSample,
                              oimg.sampleType);

         var id = ImageWindow.activeWindow.currentView.id + "_";

         for ( var c = 0; c < oimg.numberOfChannels; ++c )
         {
            oimg.selectedChannel = cIndex[c];
            nimg.selectedChannel = c;
            var a = new Array(  oimg.width * oimg.height );
            oimg.getPixels( a );
            nimg.setPixels( a );
            a = null;
            id = id + "RGB"[cIndex[c]];
         }

         var vimg = new ImageWindow(oimg.width,
                                    oimg.height,
                                    oimg.numberOfChannels,
                                    oimg.bitsPerSample,
                                    oimg.sampleType == SampleType_Real,
                                    true,
                                    id);

         with ( vimg.mainView )
         {
            beginProcess( UndoFlag_NoSwapFile );
            image.transfer( nimg );
            endProcess();
         }

         vimg.show();
         vimg.zoomToFit();
         this.dialog.ok();
      }
   }
   this.cancel_Button = new PushButton (this);
   with ( this.cancel_Button )
   {
      text = "Cancel";
      onClick = function() {
         this.dialog.cancel();
      };
   }

   this.buttons_Sizer = new HorizontalSizer;
   with ( this.buttons_Sizer )
   {
      spacing = 4;
      add( this.cancel_Button );
      add( this.ok_Button );
   }

   this.sizer = new VerticalSizer;
   with ( this.sizer )
   {
      margin = 6;
      spacing = 2;
      add ( this.helpLabel );
      addSpacing (4);
      add ( this.R_Sizer );
      addSpacing (4);
      add ( this.G_Sizer );
      addSpacing (4);
      add ( this.B_Sizer );
      addSpacing (4);
      add ( this.buttons_Sizer );
   }
}

function UpdateIndex(numEdit, numEditList, indexList)
{
   var i = numEdit.value;
   var j = numEditList.indexOf( numEdit );
   var k = indexList[j];
   var l = indexList.indexOf( i );

   if (i != k)
   {
      indexList[j] = i;
      indexList[l] = k;
      numEditList[l].setValue( k );
   }
}

//

swap3.prototype = new Dialog;

function main()
{
   var window = ImageWindow.activeWindow;
   if ( window.isNull )
      throw new Error( "No active image" );

   console.show();
   console.writeln( "<end><cbr><br><b>" + window.currentView.fullId + "</b>" );
   console.writeln( "File "+ImageWindow.activeWindow.filePath);
   console.flush();

   console.abortEnabled = true;

   var dialog = new swap3();
   dialog.img = window.currentView.image;
   dialog.execute();
 }

main();

// ****************************************************************************
// EOF GetPixVal.js - Released 2012/10/04 12:00:00 UTC
